//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pos_car,jacob]=CL_co_ell2car(pos_ell,er,obla)
// Elliptical coordinates to cartesian coordinates
//
// Calling Sequence
// [pos_car[,jacob]] = CL_co_ell2car(pos_ell [,er,obla])
//
// Description
// <itemizedlist><listitem>
// This function converts elliptical ("geodetic") coordinates into cartesian coordinates. 
// <para> Elliptical coordinates consists in longitude, latitude and altitude with respect to 
// some reference ellipsoid. </para>
// <para> The reference ellipsoid is an ellipsoid of revolution around the Z axis. It is then 
// characterized by 2 parameters: </para>
// <para> - semi-major axis of the ellipse obtained as the intersection of the ellipsoid with any 
// plane containing the Z axis, </para>
// <para> - oblateness (or flattening) of this ellipse. </para>
// <para><inlinemediaobject><imageobject><imagedata fileref="elliptical_coord.gif"/></imageobject></inlinemediaobject></para>
// </listitem>
// <listitem>
// Notes: 
// <para> - The transformation jacobian is computed if the corresponding output argument exists. </para>
// <para> - Be careful that the 3rd elliptical coordinate is an altitude and not the distance to the planet center. </para>
// </listitem>
// </itemizedlist>
// <para><emphasis role="bold">( Last updated: 2010-02-17 )</emphasis></para>
//
// Parameters
// pos_ell : [lon;lat;alt] Position vector in elliptical ("geodetic") coordinates [rad,m] (3xN)
// er : (optional) Equatorial radius of reference ellipsoid [m] (default is %CL_eqRad)
// obla : (optional) Oblateness of reference ellipsoid (default is %CL_obla)
// pos_car : [x;y;z] Position vector in cartesian coordinates [m] (3xN)
// jacob : (optional) Transformation jacobian (3x3xN)
//
// Authors
// CNES - DCT/SB
//
// Bibliography
// 1 Mecanique Spatiale, Cnes - Cepadues Editions, Tome I, section 3.2.3 (Les reperes de l'espace et du temps, Relations entre les coordonnees)
// 2 CNES - MSLIB FORTRAN 90, Volume T (mt_geod_car)
//
// See also
// CL_co_car2ell
// CL_co_sph2car
// CL_co_car2sph
//
// Examples
// // Example 1 :
// pos_ell = [ CL_deg2rad(12.125) ; CL_deg2rad(108.2) ; 110.0];
// [pos_car,jacob1] = CL_co_ell2car(pos_ell);

// Declarations:
global %CL_eqRad %CL_obla;

// Code:

[lhs,rhs]=argn();
if ~exists('er','local') then er = %CL_eqRad; end
if exists('obl','local') then obla = obl; end // for ascending compatibility
if ~exists('obla','local')
  obla = %CL_obla;
else
  if obla>=1 then CL__error('oblatenes bigger or equal to 1'); end
end
select lhs
  case 1
    compute_jacob = %f
  case 2
    compute_jacob = %t
  else
    CL__error('check number of output arguments')
end

//get latitude, longitude and altitude
lat = pos_ell(2,:);
lon = pos_ell(1,:);
hg = pos_ell(3,:);

//e and N calculation (see ref. 1)
ex2 = obla*(2-obla)
correc = 1 - ex2.*(sin(lat)^2)  //correc>0, might be 0 if obla=1
Rphi = er./sqrt(correc) //radius at given latitude
RphiplusH = Rphi + hg

//cartesian coordinates:
x = RphiplusH.*cos(lat).*cos(lon);
y = RphiplusH.*cos(lat).*sin(lon);
z = (Rphi.*(1-ex2)+hg).*sin(lat);

pos_car = [x;y;z];

//JACOBIAN COMPUTATION
if compute_jacob
  jacob = zeros(3,3,size(pos_ell,2))

  K = (1 - ex2)./correc //K in [0,1]
  //if obla = 0 : ex2 = 0 ; correc = 1 ; Rphi = er; K = 1

  KRphiplusH = K.*Rphi + hg

  jacob(1,2,:) = - KRphiplusH .* sin(lat).*cos(lon)
  jacob(1,1,:) = - RphiplusH .* cos(lat).*sin(lon)
  jacob(1,3,:) = cos(lat) .* cos(lon)

  jacob(2,2,:) = - KRphiplusH .* sin(lat) .* sin(lon)
  jacob(2,1,:) = RphiplusH .* cos(lat) .* cos(lon)
  jacob(2,3,:) = cos(lat) .* sin(lon)

  jacob(3,2,:) = KRphiplusH .* cos(lat)
  jacob(3,1,:) = 0
  jacob(3,3,:) = sin(lat)

  if size(pos_ell,2)==1 then jacob=jacob(:,:,1); end

end

endfunction

