//  Copyright (c) CNES  2008
//
//  This software is part of CelestLab, a CNES toolbox for Scilab
//
//  This software is governed by the CeCILL  license under French law and
//  abiding by the rules of distribution of free software.  You can  use,
//  modify and/ or redistribute the software under the terms of the CeCILL
//  license as circulated by CEA, CNRS and INRIA at the following URL
//  'http://www.cecill.info'.

function [pos_car,vel_car,jacob]=CL_oe_kep2car(kep,mu)
// Keplerian to cartesian orbital elements
//
// Calling Sequence
// [pos_car,vel_car[,jacob]] = CL_oe_kep2car(kep[,mu])
//
// Description
// <itemizedlist><listitem>
// Converts classical keplerian orbital elements to position and velocity  
// for an elliptic, hyperbolic or parabolic orbit.
// <para>The transformation jacobian is optionally computed.</para>
// <para>See <link linkend="OrbitalElements">Orbital elements</link> for more details</para>
// </listitem>
// </itemizedlist>
// <para><emphasis role="bold">( Last updated: 2010-02-17 )</emphasis></para>
//
// Parameters
// kep: classical keplerian parameters [sma;ecc;inc;pom;raan;anm] [m,rad] (6xN)
// mu : (optional) gravitational constant. [m^3/s^2] (default value is %CL_mu)
// pos_car: position [X;Y;Z] [m] (3xN)
// vel_car: velocity [Vx;Vy;Vz] [m/s] (3xN)
// jacob: (optional) transformation jacobian d(X,Y,Z,Vx,Vy,Vz)/d(sma,ecc,inc,pom,raan,anm) (6x6xN)
//
// Bibliography
// 1 CNES - MSLIB FORTRAN 90, Volume V (mv_kep_car)
//
// See also
// CL_oe_car2kep
// CL_oe_kep2cirEqua
//
// Authors
// CNES - DCT/SB
//
// Examples
// // Example 1
// kep=[24464560,0.7311,0.122138,3.10686,1.00681,0.048363]';
// [pos_car,vel_car]=CL_oe_kep2car(kep);
// [kep2]=CL_oe_car2kep(pos_car,vel_car);

// Declarations:
global %CL_epsilon %CL_mu;

// Code:

[lhs,rhs]=argn();
if ~exists('mu','local') then mu=%CL_mu; end
select lhs
  case 3
    compute_jacob=%t
  case 2
    compute_jacob=%f
  case 1
    compute_jacob=%f
  else
    CL__error('check number of output arguments')
end

aoup = kep(1,:)
ecc = kep(2,:)
ai = kep(3,:)

eps100 = 100*%eps

pos_car = zeros(3,size(kep,2));
vel_car = pos_car;

//data tests
i_aoup = find(aoup<=eps100) //semi major axis or parabola parameter <=0
if i_aoup~=[] then CL__error('semi major axis or parabola parameter <=0'); end

i_ecc = find(ecc<0)
if i_ecc~=[] then CL__error('negative eccentricity'); end

i_ai = find( (ai<0)|(ai>%pi) )
if i_ai~=[] then CL__error('negative inclination or bigger than pi'); end

//keplerien -> cartesien conversion depending on type of orbit
//----------------------
//elliptic case
i_ellip = find( ecc <= 1-%CL_epsilon.parab )
if i_ellip~=[]
  if compute_jacob
    [pos,vel,jac] = CL__oe_kep2carEllip(kep(:,i_ellip),mu)
    jacob(:,:,i_ellip) = jac
  else
    [pos,vel] = CL__oe_kep2carEllip(kep(:,i_ellip),mu)
  end
  pos_car(:,i_ellip) = pos
  vel_car(:,i_ellip) = vel
end
//hyperbolic case
i_hyper = find( ecc >= 1+%CL_epsilon.parab )
if i_hyper~=[]
  if compute_jacob
    [pos,vel,jac] = CL__oe_kep2carHyperb(kep(:,i_hyper),mu)
    jacob(:,:,i_hyper) = jac
  else
    [pos,vel] = CL__oe_kep2carHyperb(kep(:,i_hyper),mu)
  end
  pos_car(:,i_hyper) = pos
  vel_car(:,i_hyper) = vel
end
//parabolic case
i_parab = find( ecc == 1 )
if i_parab~=[]
  if compute_jacob
    [pos,vel,jac] = CL__oe_kep2carParab(kep(:,i_parab),mu)
    jacob(:,:,i_parab) = jac
  else
    [pos,vel] = CL__oe_kep2carParab(kep(:,i_parab),mu)
  end
  pos_car(:,i_parab) = pos
  vel_car(:,i_parab) = vel
end

endfunction

